@@ -4,13 +4,12 @@ use feature qw(say);
 
 package App::GitHubPullRequest;
 {
-  $App::GitHubPullRequest::VERSION = '0.0.4';
+  $App::GitHubPullRequest::VERSION = '0.0.5';
 }
 
 # ABSTRACT: Command-line tool to query GitHub pull requests
 
-use JSON qw(decode_json);
-use Data::Dumper qw(Dumper);
+use JSON qw(decode_json encode_json);
 
 sub new {
     my ($class) = @_;
@@ -36,8 +35,13 @@ Where command is one of these:
 
   list [<state>] Show all pull requests (default)
                      state: open/closed (default: open)
-  show <number>  Show details for a specific pull request
-  patch <number> Fetch a properly formatted patch for specific pull request
+  show <number>  Show details for the specific pull request
+  patch <number> Fetch a properly formatted patch for the specific pull request
+  close <number> Close the specified pull request
+  open <number>  Reopen the specified pull request
+
+  comment <number> <text> Create a comment on the specified pull request
+
   help           Show this page
 
 EOM
@@ -105,6 +109,57 @@ sub patch {
     return 0;
 }
 
+
+sub close {
+    my ($self, $number) = @_;
+    die("Please specify a pull request number.\n") unless $number;
+    my $pr = $self->_state($number, 'closed');
+    die("Unable to close pull request $number.\n")
+        unless defined $pr;
+    say "Pull request $number now in state: " . $pr->{'state'};
+    return 0;
+}
+
+
+sub open {
+    my ($self, $number) = @_;
+    die("Please specify a pull request number.\n") unless $number;
+    my $pr = $self->_state($number, 'open');
+    die("Unable to open pull request $number.\n")
+        unless defined $pr;
+    say "Pull request $number now in state: " . $pr->{'state'};
+    return 0;
+}
+
+
+sub comment {
+    my ($self, $number, $text) = @_;
+    die("Please specify a pull request number.\n") unless $number;
+    die("Please specify some text.\n") unless $text;
+    my $remote_repo = _find_github_remote();
+    my $url = "https://api.github.com/repos/$remote_repo/issues/$number/comments";
+    my $mimetype = 'application/json';
+    my $data = encode_json({ "body" => $text });
+    my $comment = decode_json( _post_url($url, $mimetype, $data) );
+    die("Unable to add comment on pull request $number.\n")
+        unless defined $comment;
+    say "Comment added. You can view it online here:\n"
+      . $comment->{'html_url'};
+    return 0;
+}
+
+sub _state {
+    my ($self, $number, $state) = @_;
+    croak("Please specify a pull request number") unless $number;
+    croak("Please specify a pull request state") unless $state;
+    my $remote_repo = _find_github_remote();
+    my $url = "https://api.github.com/repos/$remote_repo/pulls/$number";
+    my $mimetype = 'application/json';
+    my $data = encode_json({ "state" => $state });
+    my $pr = decode_json( _patch_url($url, $mimetype, $data) );
+    return $pr;
+}
+
 sub _fetch_comments {
     my ($self, $number) = @_;
     my $pr = $self->_fetch_one($number);
@@ -196,6 +251,7 @@ sub _fetch_url {
         }
     }
 
+    # Fetch information
     my $content = qx{curl -s -w '\%{http_code}' $credentials "$url"};
     my $rc = $? >> 8; # see perldoc perlvar $? entry for details
     die("curl failed to fetch $url with code $rc.\n") if $rc != 0;
@@ -206,6 +262,80 @@ sub _fetch_url {
     return $content;
 }
 
+# Send a PATCH request to a URL
+# If URL starts with https://api.github.com/, use github user+password from
+# your ~/.gitconfig
+sub _patch_url {
+    my ($url, $mimetype, $data) = @_;
+    croak("Please specify a URL") unless $url;
+    croak("Please specify a mimetype") unless $mimetype;
+    croak("Please specify some data") unless $data;
+
+    # See if we should use credentials
+    my $credentials = "";
+    if ( $url =~ m{^https://api.github.com/} ) {
+        my $user = qx{git config github.user};
+        my $password = qx{git config github.password};
+        chomp $user;
+        chomp $password;
+        die("You must set 'git config github.user' and 'git config github.password' to modify pull requests.\n")
+            unless $user and $password;
+        $credentials = qq{-u "$user:$password"};
+    }
+
+    # Prepare modification request
+    my $mime = qq{-H "Content-Type: $mimetype"};
+    $data =~ s{'}{\\'}; # Escape single quotes
+    my $datatosend = qq{-d '$data'};
+
+    # Send modification request
+    my $content = qx{curl -s -w '\%{http_code}' -X PATCH $credentials $mime $datatosend "$url"};
+    my $rc = $? >> 8; # see perldoc perlvar $? entry for details
+    die("curl failed to patch $url with code $rc.\n") if $rc != 0;
+    my $code = substr($content, -3, 3, '');
+    if ( $code >= 400 ) {
+        die("Patching URL $url failed with code $code:\n$content\n");
+    }
+    return $content;
+}
+
+# Send a POST request to a URL
+# If URL starts with https://api.github.com/, use github user+password from
+# your ~/.gitconfig
+sub _post_url {
+    my ($url, $mimetype, $data) = @_;
+    croak("Please specify a URL") unless $url;
+    croak("Please specify a mimetype") unless $mimetype;
+    croak("Please specify some data") unless $data;
+
+    # See if we should use credentials
+    my $credentials = "";
+    if ( $url =~ m{^https://api.github.com/} ) {
+        my $user = qx{git config github.user};
+        my $password = qx{git config github.password};
+        chomp $user;
+        chomp $password;
+        die("You must set 'git config github.user' and 'git config github.password' to modify pull requests.\n")
+            unless $user and $password;
+        $credentials = qq{-u "$user:$password"};
+    }
+
+    # Prepare modification request
+    my $mime = qq{-H "Content-Type: $mimetype"};
+    $data =~ s{'}{\\'}; # Escape single quotes
+    my $datatosend = qq{-d '$data'};
+
+    # Send modification request
+    my $content = qx{curl -s -w '\%{http_code}' -X POST $credentials $mime $datatosend "$url"};
+    my $rc = $? >> 8; # see perldoc perlvar $? entry for details
+    die("curl failed to post to $url with code $rc.\n") if $rc != 0;
+    my $code = substr($content, -3, 3, '');
+    if ( $code >= 400 ) {
+        die("Posting to URL $url failed with code $code:\n$content\n");
+    }
+    return $content;
+}
+
 1;
 
 __END__
@@ -220,7 +350,7 @@ App::GitHubPullRequest - Command-line tool to query GitHub pull requests
 
 =head1 VERSION
 
-version 0.0.4
+version 0.0.5
 
 =head1 SYNOPSIS
 
@@ -228,6 +358,9 @@ version 0.0.4
     $ prq list closed # not shown by default
     $ prq show 7      # also includes comments
     $ prq patch 7     # can be piped to colordiff if you like colors
+    $ prq close 7
+    $ prq open 7
+    $ prq comment 7 "This is good stuff!"
     $ prq help
 
 =head1 INSTALLATION
@@ -258,6 +391,18 @@ comments.
 
 Shows the patch associated with the specified pull request number.
 
+=head2 close <number>
+
+Closes the specified pull request number.
+
+=head2 open <number>
+
+Reopens the specified pull request number.
+
+=head2 comment <number> <text>
+
+Creates a comment on the specified pull request with the specified text.
+
 =head1 METHODS
 
 =head2 run(@args)
